@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2018-2023 Codership Oy <info@codership.com>
+// Copyright (C) 2018-2019 Codership Oy <info@codership.com>
 //
 
 #include <wsrep_api.h>
@@ -64,12 +64,10 @@ static const char* Defaults[] =
     "gcache.recover",              "yes",
     "gcache.size",                 "128M",
     "gcomm.thread_prio",           "",
-    "gcs.check_appl_proto",        "1",
     "gcs.fc_debug",                "0",
     "gcs.fc_factor",               "1.0",
     "gcs.fc_limit",                "16",
     "gcs.fc_master_slave",         "no",
-    "gcs.fc_single_primary",       "no",
     "gcs.max_packet_size",         "64500",
     "gcs.max_throttle",            "0.25",
 #if (GU_WORDSIZE == 32)
@@ -104,20 +102,18 @@ static const char* Defaults[] =
     "repl.commit_order",           "3",
     "repl.key_format",             "FLAT8",
     "repl.max_ws_size",            "2147483647",
-    "repl.proto_max",              "11",
+    "repl.proto_max",              "10",
 #ifdef GU_DBUG_ON
     "signal",                      "",
 #endif
     "socket.checksum",             "2",
     "socket.recv_buf_size",        "auto",
     "socket.send_buf_size",        "auto",
-//  "socket.dynamic",              no default,
 //  "socket.ssl",                  no default,
 //  "socket.ssl_cert",             no default,
 //  "socket.ssl_cipher",           no default,
 //  "socket.ssl_compression",      no default,
 //  "socket.ssl_key",              no default,
-//  "socket.ssl_reload"            no default,
     NULL
 };
 
@@ -131,8 +127,8 @@ fill_in_expected(DefaultsMap& map, const char* def_list[])
         std::pair<std::string, std::string> param(def_list[i], def_list[i+1]);
         DefaultsMap::iterator it(map.insert(param).first);
 
-        ck_assert_msg(it != map.end(), "Failed to insert KV pair: %s = %s",
-                      param.first.c_str(), param.second.c_str());
+        fail_if(it == map.end(), "Failed to insert KV pair: %s = %s",
+                param.first.c_str(), param.second.c_str());
     }
 }
 
@@ -150,8 +146,8 @@ fill_in_real(DefaultsMap& map, wsrep_t& provider)
                                                           kv_pairs[i].second);
         DefaultsMap::iterator it(map.insert(trimmed).first);
 
-        ck_assert_msg(it != map.end(), "Failed to insert KV pair: %s = %s",
-                      trimmed.first.c_str(), trimmed.second.c_str());
+        fail_if(it == map.end(), "Failed to insert KV pair: %s = %s",
+                trimmed.first.c_str(), trimmed.second.c_str());
     }
 }
 
@@ -226,7 +222,7 @@ recv_func(void* ctx)
     wsrep_t& provider(c->provider_);
 
     wsrep_status_t const ret(provider.recv(&provider, NULL));
-    ck_assert_msg(WSREP_OK == ret, "recv() returned %d", ret);
+    fail_if(WSREP_OK != ret, "recv() returned %d", ret);
 
     return NULL;
 }
@@ -240,7 +236,7 @@ START_TEST(defaults)
     app_ctx ctx;
     wsrep_t& provider(ctx.provider_);
     int ret = wsrep_status_t(wsrep_loader(&provider));
-    ck_assert(WSREP_OK == ret);
+    fail_if(WSREP_OK != ret);
 
     struct wsrep_init_args init_args =
         {
@@ -274,11 +270,11 @@ START_TEST(defaults)
             synced_cb,// wsrep_synced_cb_t synced_cb
         };
     ret = provider.init(&provider, &init_args);
-    ck_assert(WSREP_OK == ret);
+    fail_if(WSREP_OK != ret);
 
     /* some defaults are set only on connection attmept */
     ret = provider.connect(&provider, "cluster_name", "gcomm://", "", false);
-    ck_assert_msg(WSREP_OK == ret, "connect() returned %d", ret);
+    fail_if(WSREP_OK != ret, "connect() returned %d", ret);
 
     fill_in_real(real_defaults, provider);
     mark_point();
@@ -301,13 +297,13 @@ START_TEST(defaults)
         mark_point();
 
         ret = provider.disconnect(&provider);
-        ck_assert_msg(WSREP_OK == ret, "disconnect() returned %d", ret);
+        fail_if(WSREP_OK != ret, "disconnect() returned %d", ret);
 
         ret = gu_thread_join(recv_thd, NULL);
-        ck_assert_msg(0 == ret, "Could not join thread: %d (%s)",
-                      ret, strerror(ret));
+        fail_if(0 != ret, "Could not join thread: %d (%s)", ret, strerror(ret));
     }
 
+    provider.free(&provider);
     mark_point();
 
     /* cleanup files */
@@ -351,8 +347,8 @@ START_TEST(defaults)
         real_defaults.erase(real++);
     }
 
-    ck_assert_msg(err.str().empty(), "Defaults discrepancies detected:\n%s",
-                  err.str().c_str());
+    fail_if (!err.str().empty(), "Defaults discrepancies detected:\n%s",
+             err.str().c_str());
 }
 END_TEST
 
