@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2016 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2018 Codership Oy <info@codership.com>
  */
 
 #include "gcache_rb_store.hpp"
@@ -47,6 +47,7 @@ namespace gcache
                             size_t             size,
                             seqno2ptr_t&       seqno2ptr,
                             gu::UUID&          gid,
+                            int const          dbg,
                             bool const         recover)
     :
         fd_        (name, check_size(size)),
@@ -65,8 +66,10 @@ namespace gcache
         size_trail_(0),
 //        mallocs_   (0),
 //        reallocs_  (0),
+        debug_     (dbg & DEBUG),
         open_      (true)
     {
+        assert((uintptr_t(start_) % MemOps::ALIGNMENT) == 0);
         constructor_common ();
         open_preamble(recover);
         BH_clear (BH_cast(next_));
@@ -104,20 +107,21 @@ namespace gcache
             if (gu_likely (BH_is_released(bh)))
             {
                 seqno2ptr_.erase (j);
-                empty_buffer(bh);
 
                 switch (bh->store)
                 {
-                case BUFFER_IN_RB:  discard(bh); break;
+                case BUFFER_IN_RB:
+                    discard(bh);
+                    break;
                 case BUFFER_IN_MEM:
                 {
-                    MemStore* const ms(static_cast<MemStore*>(bh->ctx));
+                    MemStore* const ms(static_cast<MemStore*>(BH_ctx(bh)));
                     ms->discard(bh);
                     break;
                 }
                 case BUFFER_IN_PAGE:
                 {
-                    Page*      const page (static_cast<Page*>(bh->ctx));
+                    Page*      const page (static_cast<Page*>(BH_ctx(bh)));
                     PageStore* const ps   (PageStore::page_store(page));
                     ps->discard(bh);
                     break;
@@ -140,6 +144,7 @@ namespace gcache
     BufferHeader*
     RingBuffer::get_new_buffer (size_type const size)
     {
+        assert((size % MemOps::ALIGNMENT) == 0);
         assert_size_free();
 
         BH_assert_clear(BH_cast(next_));
@@ -232,6 +237,7 @@ namespace gcache
 #endif
 
     found_space:
+        assert((uintptr_t(ret) % MemOps::ALIGNMENT) == 0);
         size_used_ += size;
         assert (size_used_ <= size_cache_);
         assert (size_free_ >= size);
@@ -240,12 +246,11 @@ namespace gcache
         BufferHeader* const bh(BH_cast(ret));
         bh->size    = size;
         bh->seqno_g = SEQNO_NONE;
-        bh->seqno_d = SEQNO_ILL;
         bh->flags   = 0;
         bh->store   = BUFFER_IN_RB;
-        bh->ctx     = this;
-
+        bh->ctx     = reinterpret_cast<BH_ctx_t>(this);
         next_ = ret + size;
+        assert((uintptr_t(next_) % MemOps::ALIGNMENT) == 0);
         assert (next_ + sizeof(BufferHeader) <= end_);
         BH_clear (BH_cast(next_));
         assert_sizes();
@@ -390,28 +395,25 @@ namespace gcache
 
         if (size_cache_ == size_free_) return;
 
-        /* Find the last seqno'd RB buffer. It is likely to be close to the
-         * end of released buffers chain. */
+        /* Invalidate seqnos for all ordered buffers (so that they can't be
+         * recovered on restart. Also find the last seqno'd RB buffer. */
         BufferHeader* bh(0);
 
-        for (seqno2ptr_t::reverse_iterator r(seqno2ptr_.rbegin());
-             r != seqno2ptr_.rend(); ++r)
+        for (seqno2ptr_t::iterator i(seqno2ptr_.begin());
+             i != seqno2ptr_.end(); ++i)
         {
-            BufferHeader* const b(ptr2BH(r->second));
+            BufferHeader* const b(ptr2BH(i->second));
             if (BUFFER_IN_RB == b->store)
             {
 #ifndef NDEBUG
                 if (!BH_is_released(b))
                 {
-                    log_fatal << "Buffer "
-                              << reinterpret_cast<const void*>(r->second)
-                              << ", seqno_g " << b->seqno_g << ", seqno_d "
-                              << b->seqno_d << " is not released.";
+                    log_fatal << "Buffer " << b << " is not released.";
                     assert(0);
                 }
 #endif
+                b->seqno_g = SEQNO_NONE;
                 bh = b;
-                break;
             }
         }
 
@@ -535,7 +537,6 @@ namespace gcache
     void
     RingBuffer::write_preamble(bool const synced)
     {
-        static int const VERSION(1);
         uint8_t* const preamble(reinterpret_cast<uint8_t*>(preamble_));
 
         std::ostringstream os;
@@ -578,8 +579,8 @@ namespace gcache
     void
     RingBuffer::open_preamble(bool const do_recover)
     {
+        int version(0); // used only for recovery on upgrade
         uint8_t* const preamble(reinterpret_cast<uint8_t*>(preamble_));
-        int version(0);
         long long seqno_max(SEQNO_ILL);
         long long seqno_min(SEQNO_ILL);
         off_t offset(-1);
@@ -616,13 +617,22 @@ namespace gcache
            version = 0;
         }
 
-        if (offset < -1 || (preamble + offset + sizeof(BufferHeader)) > end_)
+        if (offset < -1 ||
+            (preamble + offset + sizeof(BufferHeader)) > end_ ||
+            (version >= 2 && offset >= 0 && (offset % MemOps::ALIGNMENT)))
         {
            log_warn << "Bogus offset in GCache ring buffer preamble: "
                     << offset << ". Assuming unknown.";
            offset = -1;
         }
 
+        log_info << "GCache DEBUG: opened preamble:"
+                 << "\nVersion: " << version
+                 << "\nUUID: " << gid_
+                 << "\nSeqno: " << seqno_min << " - " << seqno_max
+                 << "\nOffset: " << offset
+                 << "\nSynced: " << synced;
+
         if (do_recover)
         {
             if (gid_ != gu::UUID())
@@ -632,7 +642,7 @@ namespace gcache
 
                 try
                 {
-                    recover(offset - (start_ - preamble));
+                    recover(offset - (start_ - preamble), version);
                 }
                 catch (gu::Exception& e)
                 {
@@ -658,7 +668,7 @@ namespace gcache
     }
 
     int64_t
-    RingBuffer::scan(off_t const offset)
+    RingBuffer::scan(off_t const offset, int const scan_step)
     {
         int segment_scans(0);
         int64_t seqno_max(-1);
@@ -672,6 +682,8 @@ namespace gcache
         /* start at offset (first segment) if we know it and it is valid */
         if (offset >= 0)
         {
+            assert(0 == (offset % scan_step));
+
             if (start_ + offset + sizeof(BufferHeader) < segment_end)
                 /* we know exaclty where the first segment starts */
                 segment_start = start_ + offset;
@@ -697,8 +709,10 @@ namespace gcache
 
             while (GCACHE_SCAN_BUFFER_TEST)
             {
+                assert((uintptr_t(bh) % scan_step) == 0);
+
                 bh->flags |= BUFFER_RELEASED;
-                bh->ctx    = this;
+                bh->ctx    = uint64_t(this);
 
                 int64_t const seqno_g(bh->seqno_g);
 
@@ -819,10 +833,9 @@ namespace gcache
                 while (!GCACHE_SCAN_BUFFER_TEST &&
                        ptr + sizeof(BufferHeader) < end_)
                 {
-                    progress.update(1);
-                    ptr += 1;
+                    progress.update(scan_step);
+                    ptr += scan_step;
                     bh = BH_cast(ptr);
-                    /* it would have been nice to use alignment of 8 or so */
                 }
 
                 if (GCACHE_SCAN_BUFFER_TEST)
@@ -830,7 +843,7 @@ namespace gcache
                     segment_start = ptr;
                     first_ = segment_start;
                 }
-                else if (ptr + sizeof(BufferHeader) == end_)
+                else if (ptr + sizeof(BufferHeader) >= end_)
                 {
                     /* perhaps it was a single segment starting at start_ */
                     first_ = start_;
@@ -878,12 +891,12 @@ namespace gcache
     }
 
     void
-    RingBuffer::recover(off_t const offset)
+    RingBuffer::recover(off_t const offset, int version)
     {
         static const char* const diag_prefix = "Recovering GCache ring buffer: ";
 
         /* scan the buffer and populate seqno2ptr map */
-        int64_t const lower(scan(offset));
+        int64_t const lower(scan(offset, version > 0 ? MemOps::ALIGNMENT : 1));
 
         if (!seqno2ptr_.empty())
         {
@@ -965,7 +978,6 @@ namespace gcache
             {
                 if (gu_likely(bh->size) > 0)
                 {
-                    assert(bh->seqno_d >= SEQNO_ILL);
                     assert(bh->size > sizeof(BufferHeader));
 
                     if (bh->seqno_g > 0) last_bh = bh;
@@ -979,6 +991,22 @@ namespace gcache
             }
             next_ = reinterpret_cast<uint8_t*>(BH_next(last_bh));
 
+            /* Even if previous buffers were not aligned, make sure from
+             * now on they are - adjust next_ pointer and last buffer size */
+            if (uintptr_t(next_) % MemOps::ALIGNMENT)
+            {
+                uint8_t* const n(MemOps::align_ptr(next_));
+                assert(n > next_);
+                size_type const size_diff(n - next_);
+                assert(size_diff < MemOps::ALIGNMENT);
+                assert(last_bh->size > 0);
+                last_bh->size += size_diff;
+                next_ = n;
+                assert(BH_next(last_bh) == BH_cast(next_));
+            }
+            assert((uintptr_t(next_) % MemOps::ALIGNMENT) == 0);
+            BH_clear(BH_cast(next_));
+
             /* at this point we must have at least one seqno'd buffer */
             assert(next_ != first_);
 
