@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2023 Codership Oy <info@codership.com>
+ * Copyright (C) 2009-2019 Codership Oy <info@codership.com>
  */
 
 #ifndef EVS_MESSAGE2_HPP
@@ -141,8 +141,6 @@ public:
         EVS_T_DELAYED_LIST = 7 /*!< Evict list message */
     };
 
-    static const size_t num_message_types = EVS_T_DELAYED_LIST + 1;
-
     typedef std::map<UUID, uint8_t> DelayedList;
 
     static const uint8_t F_MSG_MORE = 0x1; /*!< Sender has more messages to send  */
@@ -290,17 +288,7 @@ public:
      */
     gu::datetime::Date tstamp() const { return tstamp_; }
 
-    /* Read type from message buffer. */
-    static Type get_type(const gu::byte_t* buf, size_t buflen, size_t offset);
-
-    /* Unserialize common header. */
-    size_t unserialize_common(const gu::byte_t* buf, size_t buflen,
-                              size_t offset);
-
-    /* Unserialize message. */
-    virtual size_t unserialize(const gu::byte_t* buf, size_t buflen,
-                               size_t offset)
-        = 0;
+    size_t unserialize(const gu::byte_t* buf, size_t buflen, size_t offset);
 
     bool operator==(const Message& cmp) const;
 
@@ -454,7 +442,8 @@ public:
     void set_aru_seq(const seqno_t as) { aru_seq_ = as; }
 
     size_t serialize(gu::byte_t* buf, size_t buflen, size_t offset) const;
-    size_t unserialize(const gu::byte_t* buf, size_t buflen, size_t offset) override;
+    size_t unserialize(const gu::byte_t* buf, size_t buflen, size_t offset,
+                       bool skip_header = false);
     size_t serial_size() const;
 
 };
@@ -513,8 +502,8 @@ public:
                 fifo_seq)
     { }
     size_t serialize(gu::byte_t* buf, size_t buflen, size_t offset) const;
-    size_t unserialize(const gu::byte_t* buf, size_t buflen,
-                       size_t offset) override;
+    size_t unserialize(const gu::byte_t* buf, size_t buflen, size_t offset,
+                       bool skip_header = false);
     size_t serial_size() const;
 };
 
@@ -546,8 +535,8 @@ public:
                 range)
     { }
     size_t serialize(gu::byte_t* buf, size_t buflen, size_t offset) const;
-    size_t unserialize(const gu::byte_t* buf, size_t buflen,
-                       size_t offset) override;
+    size_t unserialize(const gu::byte_t* buf, size_t buflen, size_t offset,
+                       bool skip_header = false);
     size_t serial_size() const;
 };
 
@@ -578,8 +567,8 @@ public:
                 node_list)
     { }
     size_t serialize(gu::byte_t* buf, size_t buflen, size_t offset) const;
-    size_t unserialize(const gu::byte_t* buf, size_t buflen,
-                       size_t offset) override;
+    size_t unserialize(const gu::byte_t* buf, size_t buflen, size_t offset,
+                       bool skip_header = false);
     size_t serial_size() const;
 };
 
@@ -611,8 +600,8 @@ public:
                 node_list)
     { }
     size_t serialize(gu::byte_t* buf, size_t buflen, size_t offset) const;
-    size_t unserialize(const gu::byte_t* buf, size_t buflen,
-                       size_t offset) override;
+    size_t unserialize(const gu::byte_t* buf, size_t buflen, size_t offset,
+                       bool skip_header = false);
     size_t serial_size() const;
 };
 
@@ -640,8 +629,8 @@ public:
                 flags)
     { }
     size_t serialize(gu::byte_t* buf, size_t buflen, size_t offset) const;
-    size_t unserialize(const gu::byte_t* buf, size_t buflen,
-                       size_t offset) override;
+    size_t unserialize(const gu::byte_t* buf, size_t buflen, size_t offset,
+                       bool skip_header = false);
     size_t serial_size() const;
 };
 
@@ -671,8 +660,8 @@ public:
     const DelayedList& delayed_list() const { return delayed_list_; }
 
     size_t serialize(gu::byte_t* buf, size_t buflen, size_t offset) const;
-    size_t unserialize(const gu::byte_t* buf, size_t buflen,
-                       size_t offset) override;
+    size_t unserialize(const gu::byte_t* buf, size_t buflen, size_t offset,
+                       bool skip_header = false);
     size_t serial_size() const;
     bool operator==(const DelayedListMessage& cmp) const
     {
