@@ -118,7 +118,7 @@ gcomm::GMCast::GMCast(Protonet& net, const gu::URI& uri,
         param<int>(conf_, uri,
                    Conf::GMCastMaxInitialReconnectAttempts,
                    gu::to_string(max_retry_cnt_))),
-    next_check_   (gu::datetime::Date::now())
+    next_check_   (gu::datetime::Date::monotonic())
 {
     log_info << "GMCast version " << version_;
 
@@ -444,8 +444,8 @@ void gcomm::GMCast::blacklist(const gmcast::Proto* proto)
     pending_addrs_.erase(proto->remote_addr());
     addr_blacklist_.insert(std::make_pair(
                                proto->remote_addr(),
-                               AddrEntry(gu::datetime::Date::now(),
-                                         gu::datetime::Date::now(),
+                               AddrEntry(gu::datetime::Date::monotonic(),
+                                         gu::datetime::Date::monotonic(),
                                          proto->remote_uuid())));
 }
 
@@ -480,7 +480,8 @@ bool gcomm::GMCast::is_not_own_and_duplicate_exists(
 void gcomm::GMCast::erase_proto(gmcast::ProtoMap::iterator i)
 {
     Proto* p(ProtoMap::value(i));
-    std::set<Socket*>::iterator si(relay_set_.find(p->socket().get()));
+    RelayEntry e(p, p->socket().get());
+    RelaySet::iterator si(relay_set_.find(e));
     if (si != relay_set_.end())
     {
         relay_set_.erase(si);
@@ -627,13 +628,13 @@ void gcomm::GMCast::gmcast_forget(const UUID& uuid,
             }
             ae.set_max_retries(0);
             ae.set_retry_cnt(1);
-            gu::datetime::Date now(gu::datetime::Date::now());
+            gu::datetime::Date now(gu::datetime::Date::monotonic());
             // Don't reduce next reconnect time if it is set greater than
             // requested
             if ((now + wait_period > ae.next_reconnect()) ||
                 (ae.next_reconnect() == gu::datetime::Date::max()))
             {
-                ae.set_next_reconnect(gu::datetime::Date::now() + wait_period);
+                ae.set_next_reconnect(gu::datetime::Date::monotonic() + wait_period);
             }
             else
             {
@@ -702,8 +703,8 @@ void gcomm::GMCast::handle_established(Proto* est)
         remote_addrs_.erase(i);
         i = remote_addrs_.insert_unique(
             make_pair(est->remote_addr(),
-                      AddrEntry(gu::datetime::Date::now(),
-                                gu::datetime::Date::now(),
+                      AddrEntry(gu::datetime::Date::monotonic(),
+                                gu::datetime::Date::monotonic(),
                                 est->remote_uuid())));
     }
 
@@ -810,7 +811,7 @@ void gcomm::GMCast::handle_failed(Proto* failed)
             AddrEntry& ae(AddrList::value(i));
             ae.set_retry_cnt(ae.retry_cnt() + 1);
 
-            gu::datetime::Date rtime = gu::datetime::Date::now() + gu::datetime::Period("PT1S");
+            gu::datetime::Date rtime = gu::datetime::Date::monotonic() + gu::datetime::Period("PT1S");
             log_debug << self_string()
                       << " setting next reconnect time to "
                       << rtime << " for " << remote_addr;
@@ -850,8 +851,8 @@ void gcomm::GMCast::insert_address (const std::string& addr,
     }
 
     if (alist.insert(make_pair(addr,
-                               AddrEntry(gu::datetime::Date::now(),
-                                         gu::datetime::Date::now(), uuid))).second == false)
+                               AddrEntry(gu::datetime::Date::monotonic(),
+                                         gu::datetime::Date::monotonic(), uuid))).second == false)
     {
         log_warn << "Duplicate entry: " << addr;
     }
@@ -972,7 +973,7 @@ void gcomm::GMCast::update_addresses()
 
                     // Add some randomness for first reconnect to avoid
                     // simultaneous connects
-                    gu::datetime::Date rtime(gu::datetime::Date::now());
+                    gu::datetime::Date rtime(gu::datetime::Date::monotonic());
 
                     rtime = rtime + ::rand() % (100*gu::datetime::MSec);
                     ae.set_next_reconnect(rtime);
@@ -991,25 +992,25 @@ void gcomm::GMCast::update_addresses()
     if (mcast_)
     {
         log_debug << mcast_addr_;
-        local_segment.push_back(mcast_.get());
+        local_segment.push_back(RelayEntry(0, mcast_.get()));
     }
 
     self_index_ = 0;
     for (ProtoMap::const_iterator i(proto_map_->begin()); i != proto_map_->end();
          ++i)
     {
-        const Proto& p(*ProtoMap::value(i));
+        Proto* p(ProtoMap::value(i));
 
-        log_debug << "Proto: " << p;
+        log_debug << "Proto: " << *p;
 
-        if (p.remote_segment() == segment_)
+        if (p->remote_segment() == segment_)
         {
-            if (p.state() == Proto::S_OK &&
-                (p.mcast_addr() == "" ||
-                 p.mcast_addr() != mcast_addr_))
+            if (p->state() == Proto::S_OK &&
+                (p->mcast_addr() == "" ||
+                 p->mcast_addr() != mcast_addr_))
             {
-                local_segment.push_back(p.socket().get());
-                if (p.remote_uuid() < uuid())
+                local_segment.push_back(RelayEntry(p, p->socket().get()));
+                if (p->remote_uuid() < uuid())
                 {
                     ++self_index_;
                 }
@@ -1017,10 +1018,10 @@ void gcomm::GMCast::update_addresses()
         }
         else
         {
-            if (p.state() == Proto::S_OK)
+            if (p->state() == Proto::S_OK)
             {
-                Segment& remote_segment(segment_map_[p.remote_segment()]);
-                remote_segment.push_back(p.socket().get());
+                Segment& remote_segment(segment_map_[p->remote_segment()]);
+                remote_segment.push_back(RelayEntry(p, p->socket().get()));
             }
         }
     }
@@ -1039,7 +1040,7 @@ void gcomm::GMCast::reconnect()
 
     /* Loop over known remote addresses and connect if proto entry
      * does not exist */
-    gu::datetime::Date now = gu::datetime::Date::now();
+    gu::datetime::Date now = gu::datetime::Date::monotonic();
     AddrList::iterator i, i_next;
 
     for (i = pending_addrs_.begin(); i != pending_addrs_.end(); i = i_next)
@@ -1158,27 +1159,44 @@ void gcomm::GMCast::check_liveness()
     std::set<UUID> live_uuids;
 
     // iterate over proto map and mark all timed out entries as failed
-    gu::datetime::Date now(gu::datetime::Date::now());
+    gu::datetime::Date now(gu::datetime::Date::monotonic());
     for (ProtoMap::iterator i(proto_map_->begin()); i != proto_map_->end(); )
     {
+        // Store next iterator into temporary, handle_failed() may remove
+        // the entry proto_map_.
         ProtoMap::iterator i_next(i);
         ++i_next;
         Proto* p(ProtoMap::value(i));
         if (p->state() > Proto::S_INIT &&
             p->state() < Proto::S_FAILED &&
-            p->tstamp() + peer_timeout_ < now)
+            p->recv_tstamp() + peer_timeout_ < now)
         {
+            gcomm::SocketStats stats(p->socket()->stats());
             log_info << self_string()
                      << " connection to peer "
                      << p->remote_uuid() << " with addr "
                      << p->remote_addr()
-                     << " timed out, no messages seen in " << peer_timeout_;
+                     << " timed out, no messages seen in " << peer_timeout_
+                     << ", socket stats: "
+                     << stats;
             p->set_state(Proto::S_FAILED);
             handle_failed(p);
         }
         else if (p->state() == Proto::S_OK)
         {
-            if (p->tstamp() + peer_timeout_*2/3 < now)
+            gcomm::SocketStats stats(p->socket()->stats());
+            if (stats.send_queue_length >= 1024)
+            {
+                log_debug << self_string()
+                          << " socket send queue to "
+                          << " peer "
+                          << p->remote_uuid() << " with addr "
+                          << p->remote_addr()
+                          << ", socket stats: "
+                          << stats;
+            }
+            if ((p->recv_tstamp() + peer_timeout_*2/3 < now) ||
+                (p->send_tstamp() + peer_timeout_*1/3 < now))
             {
                 p->send_keepalive();
             }
@@ -1256,7 +1274,7 @@ void gcomm::GMCast::check_liveness()
             log_debug << "relay set maxel :" << *p << " count: "
                       << CmpUuidCounts(nonlive_uuids, segment_).count(p);
 
-            relay_set_.insert(p->socket().get());
+            relay_set_.insert(RelayEntry(p, p->socket().get()));
             const LinkMap& lm(p->link_map());
             for (LinkMap::const_iterator lm_i(lm.begin()); lm_i != lm.end();
                  ++lm_i)
@@ -1277,7 +1295,7 @@ void gcomm::GMCast::check_liveness()
 
 gu::datetime::Date gcomm::GMCast::handle_timers()
 {
-    const gu::datetime::Date now(gu::datetime::Date::now());
+    const gu::datetime::Date now(gu::datetime::Date::monotonic());
 
     if (now >= next_check_)
     {
@@ -1290,14 +1308,18 @@ gu::datetime::Date gcomm::GMCast::handle_timers()
 }
 
 
-void send(gcomm::Socket* s, gcomm::Datagram& dg)
+void gcomm::GMCast::send(const RelayEntry& re, int segment, gcomm::Datagram& dg)
 {
     int err;
-    if ((err = s->send(dg)) != 0)
+    if ((err = re.socket->send(segment, dg)) != 0)
     {
-        log_debug << "failed to send to " << s->remote_addr()
+        log_debug << "failed to send to " << re.socket->remote_addr()
                   << ": (" << err << ") " << strerror(err);
     }
+    else
+    {
+        re.proto->set_send_tstamp(gu::datetime::Date::monotonic());
+    }
 }
 
 void gcomm::GMCast::relay(const Message& msg,
@@ -1317,15 +1339,16 @@ void gcomm::GMCast::relay(const Message& msg,
     if (msg.flags() & Message::F_RELAY)
     {
         gu_trace(push_header(relay_msg, relay_dg));
-        for (SegmentMap::iterator i(segment_map_.begin());
-             i != segment_map_.end(); ++i)
+        for (SegmentMap::iterator segment_i(segment_map_.begin());
+             segment_i != segment_map_.end(); ++segment_i)
         {
-            Segment& segment(i->second);
-            for (Segment::iterator j(segment.begin()); j != segment.end(); ++j)
+            Segment& segment(segment_i->second);
+            for (Segment::iterator target_i(segment.begin());
+                 target_i != segment.end(); ++target_i)
             {
-                if ((*j)->id() != exclude_id)
+                if ((*target_i).socket->id() != exclude_id)
                 {
-                    send(*j, relay_dg);
+                    send(*target_i, msg.segment_id(), relay_dg);
                 }
             }
         }
@@ -1338,10 +1361,13 @@ void gcomm::GMCast::relay(const Message& msg,
             // nodes in local segment that are not directly reachable
             relay_msg.set_flags(relay_msg.flags() | Message::F_RELAY);
             gu_trace(push_header(relay_msg, relay_dg));
-            for (std::set<Socket*>::iterator ri(relay_set_.begin());
-                 ri != relay_set_.end(); ++ri)
+            for (RelaySet::iterator relay_i(relay_set_.begin());
+                 relay_i != relay_set_.end(); ++relay_i)
             {
-                send(*ri, relay_dg);
+                if ((*relay_i).socket->id() != exclude_id)
+                {
+                    send(*relay_i, msg.segment_id(), relay_dg);
+                }
             }
             gu_trace(pop_header(relay_msg, relay_dg));
             relay_msg.set_flags(relay_msg.flags() & ~Message::F_RELAY);
@@ -1358,7 +1384,7 @@ void gcomm::GMCast::relay(const Message& msg,
         Segment& segment(segment_map_[segment_]);
         for (Segment::iterator i(segment.begin()); i != segment.end(); ++i)
         {
-            send(*i, relay_dg);
+            send(*i, msg.segment_id(), relay_dg);
         }
     }
     else
@@ -1393,7 +1419,7 @@ void gcomm::GMCast::handle_up(const void*        id,
             }
             else
             {
-                gu_trace(msg.unserialize(&dg.payload()[0],
+                gu_trace(msg.unserialize(dg.payload().data(),
                                          dg.len(),
                                          dg.offset()));
             }
@@ -1435,7 +1461,7 @@ void gcomm::GMCast::handle_up(const void*        id,
 
             try
             {
-                msg.unserialize(&dg.payload()[0], dg.len(),
+                msg.unserialize(dg.payload().data(), dg.len(),
                                 dg.offset());
             }
             catch (gu::Exception& e)
@@ -1461,7 +1487,7 @@ void gcomm::GMCast::handle_up(const void*        id,
                           Datagram(dg, dg.offset() + msg.serial_size()),
                           id);
                 }
-                p->set_tstamp(gu::datetime::Date::now());
+                p->set_recv_tstamp(gu::datetime::Date::monotonic());
                 send_up(Datagram(dg, dg.offset() + msg.serial_size()),
                         ProtoUpMeta(msg.source_uuid()));
                 return;
@@ -1470,7 +1496,7 @@ void gcomm::GMCast::handle_up(const void*        id,
             {
                 try
                 {
-                    p->set_tstamp(gu::datetime::Date::now());
+                    p->set_recv_tstamp(gu::datetime::Date::monotonic());
                     gu_trace(p->handle_message(msg));
                 }
                 catch (const gu::Exception& e)
@@ -1530,27 +1556,72 @@ void gcomm::GMCast::handle_up(const void*        id,
     }
 }
 
+static gcomm::gmcast::Proto* find_by_remote_uuid(
+    const gcomm::gmcast::ProtoMap& proto_map,
+    const gcomm::UUID& uuid)
+{
+    for (gcomm::gmcast::ProtoMap::const_iterator i(proto_map.begin());
+         i != proto_map.end(); ++i)
+    {
+        if (i->second->remote_uuid() == uuid)
+        {
+            return i->second;
+        }
+    }
+    return 0;
+}
 
 int gcomm::GMCast::handle_down(Datagram& dg, const ProtoDownMeta& dm)
 {
     Message msg(version_, Message::GMCAST_T_USER_BASE, uuid(), 1, segment_);
 
+    // If target is set and proto entry for target is found,
+    // send a direct message. Otherwise fall back for broadcast
+    // to ensure message delivery via relay
+    if (dm.target() != UUID::nil())
+    {
+        Proto* target_proto(find_by_remote_uuid(*proto_map_, dm.target()));
+        if (target_proto && target_proto->state() == Proto::S_OK)
+        {
+            gu_trace(push_header(msg, dg));
+            int err;
+            if ((err = target_proto->socket()->send(msg.segment_id(), dg)) != 0)
+            {
+                log_debug << "failed to send to "
+                          << target_proto->socket()->remote_addr()
+                          << ": (" << err << ") " << strerror(err);
+            }
+            else
+            {
+                target_proto->set_send_tstamp(gu::datetime::Date::monotonic());
+            }
+            gu_trace(pop_header(msg, dg));
+            if (err == 0)
+            {
+                return 0;
+            }
+            // In case of error fall back to broadcast
+        }
+        else
+        {
+            log_debug << "Target " << dm.target() << " proto not found";
+        }
+    }
+
     // handle relay set first, skip these peers below
     if (relay_set_.empty() == false)
     {
         msg.set_flags(msg.flags() | Message::F_RELAY);
         gu_trace(push_header(msg, dg));
-        for (std::set<Socket*>::iterator ri(relay_set_.begin());
+        for (RelaySet::iterator ri(relay_set_.begin());
              ri != relay_set_.end(); ++ri)
         {
-            send(*ri, dg);
+            send(*ri, msg.segment_id(), dg);
         }
         gu_trace(pop_header(msg, dg));
         msg.set_flags(msg.flags() & ~Message::F_RELAY);
     }
 
-
-
     for (SegmentMap::iterator si(segment_map_.begin());
          si != segment_map_.end(); ++si)
     {
@@ -1566,7 +1637,7 @@ int gcomm::GMCast::handle_down(Datagram& dg, const ProtoDownMeta& dm)
                 relay_set_.find(segment[target_idx]) == relay_set_.end())
             {
                 gu_trace(push_header(msg, dg));
-                send(segment[target_idx], dg);
+                send(segment[target_idx], msg.segment_id(), dg);
                 gu_trace(pop_header(msg, dg));
             }
         }
@@ -1581,7 +1652,7 @@ int gcomm::GMCast::handle_down(Datagram& dg, const ProtoDownMeta& dm)
                 if (relay_set_.empty() == true ||
                     relay_set_.find(*i) == relay_set_.end())
                 {
-                    send(*i, dg);
+                    send(*i, msg.segment_id(), dg);
                 }
             }
             gu_trace(pop_header(msg, dg));
@@ -1758,7 +1829,7 @@ void gcomm::GMCast::add_or_del_addr(const std::string& val)
             AddrEntry& ae(AddrList::value(ai));
             ae.set_max_retries(0);
             ae.set_retry_cnt(1);
-            ae.set_next_reconnect(gu::datetime::Date::now() + time_wait_);
+            ae.set_next_reconnect(gu::datetime::Date::monotonic() + time_wait_);
             update_addresses();
         }
         else
